﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Diagnostics;
using System.IO;

namespace SoftekBarcodeNet
{
    public class BarcodeReader : IDisposable
    {
        // External dll declarations

        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetLineJumpAT(IntPtr hbarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtCreateBarcodeInstanceT();
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtDestroyBarcodeInstanceT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetBitmapResolutionT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetBitmapResolutionT(IntPtr hBarcode, short newValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtGetDebugTraceFileT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtSetDebugTraceFileT(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] lpszNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtSetLicenseKeyT(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] lpszNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtGetLicenseKeyT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetLineJumpAT(IntPtr hBarcode, short nNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate double mtGetLowerRatioT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate double mtSetLowerRatioT(IntPtr hBarcode, double newValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetMinOccurrenceT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetMinOccurrenceT(IntPtr hBarcode, short nNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetMultipleReadT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetMultipleReadT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetNoiseReductionT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetNoiseReductionT(IntPtr hBarcode, short nNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetPageNoT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetPageNoT(IntPtr hBarcode, short nNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetPrefOccurrenceT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetPrefOccurrenceT(IntPtr hBarcode, short nNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetQuietZoneSizeT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetQuietZoneSizeT(IntPtr hBarcode, short nNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetRotationT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetRotationT(IntPtr hBarcode, short nNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate double mtGetUpperRatioT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate double mtSetUpperRatioT(IntPtr hBarcode, double newValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtGetBarStringT(IntPtr hBarcode, short index);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int mtGetBarStringAT(IntPtr hBarcode, short index, [MarshalAs(UnmanagedType.LPArray)] byte[] pBytes, int nBytes);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int mtGetRawBarStringT(IntPtr hBarcode, short index, [MarshalAs(UnmanagedType.LPArray)] byte[] pBytes, int nBytes);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtScanBarCodeT(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] file);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetPDFToolT(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] pdfTool);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtGetPDFToolT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtScanBarCodeFromDIBT(IntPtr hBarcode, IntPtr hDIB);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtScanBarCodeFromBitmapT(IntPtr hBarcode, IntPtr hBitmap);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtScanBarCodeFromStringT(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] data, uint sz);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtScanBarCodeFromBitmapAT(IntPtr hBarcode, int bmType, int bmWidth, int bmHeight, int bmWidthBytes, int bmPlanes, int bmBitsPixel, IntPtr bmBits);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadCode39T(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadCode39T(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadEAN13T(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadEAN13T(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadEAN8T(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadEAN8T(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadUPCAT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadUPCAT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadUPCET(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadUPCET(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetShowCheckDigitT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetShowCheckDigitT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtGetBarStringTypeT(IntPtr hBarcode, short index);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadCode128T(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadCode128T(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadShortCode128T(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadShortCode128T(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetCode39NeedStartStopT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetCode39NeedStartStopT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetColorThresholdT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetColorThresholdT(IntPtr hBarcode, short nNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int mtGetBarStringPageT(IntPtr hBarcode, short nBarCode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int mtGetBarStringTlxT(IntPtr hBarcode, short nBarCode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int mtGetBarStringTlyT(IntPtr hBarcode, short nBarCode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int mtGetBarStringBrxT(IntPtr hBarcode, short nBarCode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int mtGetBarStringBryT(IntPtr hBarcode, short nBarCode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadCode25T(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadCode25T(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetScanRectAT(IntPtr hBarcode, Int32 TopLeftX, Int32 TopLeftY, Int32 BottomRightX, Int32 BottomRightY, short MappingMode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadPatchCodesT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadPatchCodesT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetUseOverSamplingT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetUseOverSamplingT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetScanDirectionT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetScanDirectionT(IntPtr hBarcode, short sNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetMinLengthT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetMinLengthT(IntPtr hBarcode, short sNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetMin2DLengthT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetMin2DLengthT(IntPtr hBarcode, short sNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetShortCode128MinLengthT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetShortCode128MinLengthT(IntPtr hBarcode, short sNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetMaxLengthT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetMaxLengthT(IntPtr hBarcode, short sNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtSetTifSplitPathT(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] lpszNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtGetTifSplitPathT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtSetTifSplitValuesT(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] lpszNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtGetTifSplitValuesT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetTifSplitModeT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetTifSplitModeT(IntPtr hBarcode, short sNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadCodabarT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadCodabarT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetBarStringDirectionT(IntPtr hBarcode, short nBarCode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetConvertUPCEToEAN13T(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetConvertUPCEToEAN13T(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtScanBarCodeFromBitmapDumpT(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] strFilePath);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetSkewToleranceT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetSkewToleranceT(IntPtr hBarcode, short sNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadCode25niT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadCode25niT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetPhotometricT(IntPtr hBarcode, bool bPhotometric);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetPhotometricT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetAllowDuplicateValuesT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetAllowDuplicateValuesT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetDespeckleT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetDespeckleT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetMinSeparationT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetMinSeparationT(IntPtr hBarcode, short nNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetBarStringQualityScoreT(IntPtr hBarcode, short nBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetExtendedCode39T(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetExtendedCode39T(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetCode39ChecksumT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetCode39ChecksumT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetErrorCorrectionT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetErrorCorrectionT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadNumericT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadNumericT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtSetPatternT(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] lpszNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtGetPatternT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetPdfBppT(IntPtr hBarcode, short nNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetPdfBppT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetPdfDpiT(IntPtr hBarcode, short nNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetPdfDpiT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetMinSpaceBarWidthT(IntPtr hBarcode, short nNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetMinSpaceBarWidthT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadPDF417T(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadPDF417T(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadMicroPDF417T(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadMicroPDF417T(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetMedianFilterT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetMedianFilterT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int mtSaveResultsT(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] filePath);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int mtLoadXMLSettingsT(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] filePath, bool silent);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int mtExportXMLSettingsT(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] filePath);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int mtProcessXMLT(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] inputFilePath, [MarshalAs(UnmanagedType.LPArray)] byte[] outputFilePath, bool silent);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetCode25ChecksumT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetCode25ChecksumT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetEncodingT(IntPtr hBarcode, short nNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetEncodingT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetGammaCorrectionT(IntPtr hBarcode, short nNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetGammaCorrectionT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetSkewLineJumpT(IntPtr hBarcode, short nNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetSkewLineJumpT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadDataMatrixT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadDataMatrixT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadQRCodeT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadQRCodeT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadDatabarT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadDatabarT(IntPtr hBarcode, bool bNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetColorProcessingLevelT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetColorProcessingLevelT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetDatabarOptionsT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetDatabarOptionsT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetPdfImageOnlyT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetPdfImageOnlyT(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadCode93T(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadCode93T(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetPdfImageExtractOptionsT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetPdfImageExtractOptionsT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetPdfImageRasterOptionsT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetPdfImageRasterOptionsT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int getFunctionalityLevelT();
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int mtGetLastErrorT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int mtGetLastWarningT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int mtGetLastWinErrorT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetUseFastScanT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetUseFastScanT(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetBarcodesAtTopOfPageT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetBarcodesAtTopOfPageT(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetFastScanLineJumpT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetFastScanLineJumpT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetMaxBarcodesPerPageT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetMaxBarcodesPerPageT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetSkewedDatamatrixT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetSkewedDatamatrixT(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int mtGetTimeOutT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate int mtSetTimeOutT(IntPtr hBarcode, int value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetSkewedLinearT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetSkewedLinearT(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetReportUnreadBarcodesT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetReportUnreadBarcodesT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetShowCodabarStartStopT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetShowCodabarStartStopT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetProgressT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtScanBarCodeInBackgroundT(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] file);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtScanBarCodeFromStringInBackgroundT(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] data, uint sz);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtScanBarCodeWaitT(IntPtr hBarcode, Int32 dwMilliSeconds);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetBarCodeCountT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetBlankPageCountT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetBlankPageT(IntPtr hBarcode, short nBlankPage);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtScanBarCodeAbortT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetScanExitCodeT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetMaxThreadsT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetMaxThreadsT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetRotateBy45IfNoBarcodeT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetCode128DebugModeT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetCode128LenientT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetCode128SearchLevelT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetMedianFilterBiasT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetPdf417DebugT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetPdf417ChannelModeT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetPdfLockingT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetQrCodeReadInvertedT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetQuotedPrintableCharSetT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetWeightLongerBarcodesT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetCode39MaxRatioPcntT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetDataMatrixFinderGapToleranceT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetDataMatrixAutoUTF8T(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetPatchCodeMinOccurrenceT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetPdf417AutoUTF8T(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetPdf417MacroEscapeBackslashT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetQRCodeAutoUTF8T(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetQRCodeAutoMedianFilterT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetUseOldCode128AlgorithmT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetUseRunCacheT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetRotateBy45IfNoBarcodeT(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetCode128DebugModeT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetCode128LenientT(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetCode128SearchLevelT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetMedianFilterBiasT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetPdf417DebugT(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetPdf417ChannelModeT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetPdfLockingT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetQrCodeReadInvertedT(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetQuotedPrintableCharSetT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetWeightLongerBarcodesT(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetCode39MaxRatioPcntT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetDataMatrixFinderGapToleranceT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetDataMatrixAutoUTF8T(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetPatchCodeMinOccurrenceT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetPdf417AutoUTF8T(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetPdf417MacroEscapeBackslashT(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetQRCodeAutoUTF8T(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetQRCodeAutoMedianFilterT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetUseOldCode128AlgorithmT(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetUseRunCacheT(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetDeskewModeT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetDeskewModeT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtSetPdfPasswordT(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] lpszNewValue);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate IntPtr mtGetPdfPasswordT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetDataMatrixSearchLevelT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetDataMatrixSearchLevelT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetDataMatrixRectangleSupportT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetDataMatrixRectangleSupportT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetReadEAN13SupplementalT(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetReadEAN13SupplementalT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtSetIndexBlankPagesT(IntPtr hBarcode, bool value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate bool mtGetIndexBlankPagesT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetBlankPageMarginT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetBlankPageMarginT(IntPtr hBarcode);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtSetBlankPageStandardDeviationT(IntPtr hBarcode, short value);
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate short mtGetBlankPageStandardDeviationT(IntPtr hBarcode);

        [DllImport("gdi32.dll")]
        private static extern bool DeleteObject(IntPtr hObject);

        [DllImport("kernel32.dll")]
        private static extern IntPtr LoadLibrary([MarshalAs(UnmanagedType.LPArray)] byte[] dllPath);
        [DllImport("kernel32.dll")]
        private static extern IntPtr GetProcAddress(IntPtr hModule, string procName);
        [DllImport("kernel32.dll")]
        private static extern bool FreeLibrary(IntPtr hLibrary);
        [DllImport("kernel32.dll", SetLastError = true)]
        public static extern IntPtr LoadLibraryEx(string dllToLoad, IntPtr hFile, LoadLibraryFlags flags);

        [System.Flags]
        public enum LoadLibraryFlags : uint
        {
            DONT_RESOLVE_DLL_REFERENCES = 0x00000001,
            LOAD_IGNORE_CODE_AUTHZ_LEVEL = 0x00000010,
            LOAD_LIBRARY_AS_DATAFILE = 0x00000002,
            LOAD_LIBRARY_AS_DATAFILE_EXCLUSIVE = 0x00000040,
            LOAD_LIBRARY_AS_IMAGE_RESOURCE = 0x00000020,
            LOAD_WITH_ALTERED_SEARCH_PATH = 0x00000008,
            LOAD_LIBRARY_SEARCH_DLL_LOAD_DIR = 0x00000100,
            LOAD_LIBRARY_SEARCH_SYSTEM32 = 0x00000800,
            LOAD_LIBRARY_SEARCH_DEFAULT_DIRS = 0x00001000
        }


		public int LineJump ;
		public int MinOccurrence ;
		public int PrefOccurrence ;
		public bool MultipleRead ;
		public bool MedianFilter ;
		public int NoiseReduction ;
		public bool Despeckle ;
		public int QuietZoneSize ;
		public int SkewTolerance ;
		public int PageNo ;
		public bool ReadCode39 ;
        public bool ReadCode93;
        public bool ReadCode128;
		public bool ReadCode25 ;
		public bool ReadCode25ni ;
		public bool AllowDuplicateValues ;
		public bool ReadEAN8 ;
		public bool ReadEAN13 ;
		public bool ReadUPCA ;
		public bool ReadUPCE ;
		public bool ReadCodabar ;
		public bool ReadPatchCodes ;
		public bool ShowCheckDigit ;
		public bool Code39NeedStartStop ;
		public bool UseOverSampling ;
		public int ScanDirection ;
		public bool ConvertUPCEToEAN13 ;
		public int MinLength ;
        public int Min2DLength;
        public int MaxLength;
		public int ColorThreshold ;
		public int MinSeparation ;	
		public bool Code39Checksum ;
		public bool ExtendedCode39 ;
		public bool ErrorCorrection ;
		public bool ReadNumeric ;
		public string Pattern ;
		public string TifSplitPath ;
		public int	TifSplitMode ;
		public int	MinSpaceBarWidth ;
		public bool	LicensedCopy ;
		public bool ReadPDF417 ;
		public bool Photometric ;
		public string message ;
		public int PdfBpp ;
		public int PdfDpi ;
		public string build ;
		public int SkewLineJump ;
		public int GammaCorrection ;
		public int Encoding ;
		public bool Code25Checksum ;
		public bool ReadMicroPDF417 ;
		public int ShortCode128MinLength ;
		public bool ReadShortCode128 ;
		public string LicenseKey ;
		public bool ConvertPDFToTIF ;
		public bool ReadDataMatrix ;
        public bool ReadQRCode;
        public bool ReadDatabar;
        public bool PdfImageOnly;
		public int DatabarOptions ;
		public int ColorProcessingLevel ;
        public int PdfImageExtractOptions;
        public int PdfImageRasterOptions;
        public bool UseFastScan;
        public bool BarcodesAtTopOfPage;
        public int FastScanLineJump;
        public int MaxBarcodesPerPage;
        public bool SkewedDatamatrix;
        public int TimeOut;
        public bool SkewedLinear;
        public int ReportUnreadBarcodes;
        public int ShowCodabarStartStop;
        public bool dllLoaded;
        public int MaxThreads;
        public bool RotateBy45IfNoBarcode;
        public int Code128DebugMode;
        public bool Code128Lenient;
        public int Code128SearchLevel;
        public int MedianFilterBias;
        public bool Pdf417Debug;
        public int Pdf417ChannelMode;
        public int PdfLocking;
        public bool QrCodeReadInverted;
        public int QuotedPrintableCharSet;
        public bool WeightLongerBarcodes;
        public int Code39MaxRatioPcnt;
        public int DataMatrixFinderGapTolerance;
        public bool DataMatrixAutoUTF8;
        public int PatchCodeMinOccurrence;
        public bool Pdf417AutoUTF8;
        public bool Pdf417MacroEscapeBackslash;
        public bool QRCodeAutoUTF8;
        public short QRCodeAutoMedianFilter;
        public bool UseOldCode128Algorithm;
        public bool UseRunCache;
        public int DeskewMode;
        public string PdfPassword;
        public int DataMatrixSearchLevel;
        public int DataMatrixRectangleSupport;
        public bool ReadEAN13Supplemental;
        public bool IndexBlankPages;
        public short BlankPageMargin;
        public short BlankPageStandardDeviation;

        private IntPtr hBarcode;
        private string pdflib = "";
        private IntPtr hSoftekBarcodeDLL;
        private System.Text.UTF8Encoding encoding ;

        private bool disposed = false;

        //private string[] pdfLibs = { };
        private IntPtr pdfLibHandle;

        private mtGetLineJumpAT mtGetLineJumpA;
        private mtCreateBarcodeInstanceT mtCreateBarcodeInstance;
        private mtDestroyBarcodeInstanceT mtDestroyBarcodeInstance;
        private mtGetBitmapResolutionT mtGetBitmapResolution;
        private mtSetBitmapResolutionT mtSetBitmapResolution;
        private mtGetDebugTraceFileT mtGetDebugTraceFile;
        private mtSetDebugTraceFileT mtSetDebugTraceFile;
        private mtSetLicenseKeyT mtSetLicenseKey;
        private mtGetLicenseKeyT mtGetLicenseKey;
        private mtSetLineJumpAT mtSetLineJumpA;
        private mtGetLowerRatioT mtGetLowerRatio;
        private mtSetLowerRatioT mtSetLowerRatio;
        private mtGetMinOccurrenceT mtGetMinOccurrence;
        private mtSetMinOccurrenceT mtSetMinOccurrence;
        private mtGetMultipleReadT mtGetMultipleRead;
        private mtSetMultipleReadT mtSetMultipleRead;
        private mtGetNoiseReductionT mtGetNoiseReduction;
        private mtSetNoiseReductionT mtSetNoiseReduction;
        private mtGetPageNoT mtGetPageNo;
        private mtSetPageNoT mtSetPageNo;
        private mtGetPrefOccurrenceT mtGetPrefOccurrence;
        private mtSetPrefOccurrenceT mtSetPrefOccurrence;
        private mtGetQuietZoneSizeT mtGetQuietZoneSize;
        private mtSetQuietZoneSizeT mtSetQuietZoneSize;
        private mtGetRotationT mtGetRotation;
        private mtSetRotationT mtSetRotation;
        private mtGetUpperRatioT mtGetUpperRatio;
        private mtSetUpperRatioT mtSetUpperRatio;
        private mtGetBarStringT mtGetBarString;
        private mtGetBarStringAT mtGetBarStringA;
        private mtGetRawBarStringT mtGetRawBarString;
        private mtScanBarCodeT mtScanBarCode;
        private mtSetPDFToolT mtSetPDFTool;
        private mtGetPDFToolT mtGetPDFTool;
        private mtScanBarCodeFromDIBT mtScanBarCodeFromDIB;
        private mtScanBarCodeFromBitmapT mtScanBarCodeFromBitmap;
        private mtScanBarCodeFromStringT mtScanBarCodeFromString;
        private mtScanBarCodeFromBitmapAT mtScanBarCodeFromBitmapA;
        private mtGetReadCode39T mtGetReadCode39;
        private mtSetReadCode39T mtSetReadCode39;
        private mtGetReadEAN13T mtGetReadEAN13;
        private mtSetReadEAN13T mtSetReadEAN13;
        private mtGetReadEAN8T mtGetReadEAN8;
        private mtSetReadEAN8T mtSetReadEAN8;
        private mtGetReadUPCAT mtGetReadUPCA;
        private mtSetReadUPCAT mtSetReadUPCA;
        private mtGetReadUPCET mtGetReadUPCE;
        private mtSetReadUPCET mtSetReadUPCE;
        private mtGetShowCheckDigitT mtGetShowCheckDigit;
        private mtSetShowCheckDigitT mtSetShowCheckDigit;
        private mtGetBarStringTypeT mtGetBarStringType;
        private mtGetReadCode128T mtGetReadCode128;
        private mtSetReadCode128T mtSetReadCode128;
        private mtGetReadShortCode128T mtGetReadShortCode128;
        private mtSetReadShortCode128T mtSetReadShortCode128;
        private mtGetCode39NeedStartStopT mtGetCode39NeedStartStop;
        private mtSetCode39NeedStartStopT mtSetCode39NeedStartStop;
        private mtGetColorThresholdT mtGetColorThreshold;
        private mtSetColorThresholdT mtSetColorThreshold;
        private mtGetBarStringPageT mtGetBarStringPage;
        private mtGetBarStringTlxT mtGetBarStringTlx;
        private mtGetBarStringTlyT mtGetBarStringTly;
        private mtGetBarStringBrxT mtGetBarStringBrx;
        private mtGetBarStringBryT mtGetBarStringBry;
        private mtGetReadCode25T mtGetReadCode25;
        private mtSetReadCode25T mtSetReadCode25;
        private mtSetScanRectAT mtSetScanRectA;
        private mtGetReadPatchCodesT mtGetReadPatchCodes;
        private mtSetReadPatchCodesT mtSetReadPatchCodes;
        private mtGetUseOverSamplingT mtGetUseOverSampling;
        private mtSetUseOverSamplingT mtSetUseOverSampling;
        private mtGetScanDirectionT mtGetScanDirection;
        private mtSetScanDirectionT mtSetScanDirection;
        private mtGetMinLengthT mtGetMinLength;
        private mtSetMinLengthT mtSetMinLength;
        private mtGetMin2DLengthT mtGetMin2DLength;
        private mtSetMin2DLengthT mtSetMin2DLength;
        private mtGetShortCode128MinLengthT mtGetShortCode128MinLength;
        private mtSetShortCode128MinLengthT mtSetShortCode128MinLength;
        private mtGetMaxLengthT mtGetMaxLength;
        private mtSetMaxLengthT mtSetMaxLength;
        private mtSetTifSplitPathT mtSetTifSplitPath;
        private mtGetTifSplitPathT mtGetTifSplitPath;
        private mtSetTifSplitValuesT mtSetTifSplitValues;
        private mtGetTifSplitValuesT mtGetTifSplitValues;
        private mtGetTifSplitModeT mtGetTifSplitMode;
        private mtSetTifSplitModeT mtSetTifSplitMode;
        private mtGetReadCodabarT mtGetReadCodabar;
        private mtSetReadCodabarT mtSetReadCodabar;
        private mtGetBarStringDirectionT mtGetBarStringDirection;
        private mtGetConvertUPCEToEAN13T mtGetConvertUPCEToEAN13;
        private mtSetConvertUPCEToEAN13T mtSetConvertUPCEToEAN13;
        private mtScanBarCodeFromBitmapDumpT mtScanBarCodeFromBitmapDump;
        private mtGetSkewToleranceT mtGetSkewTolerance;
        private mtSetSkewToleranceT mtSetSkewTolerance;
        private mtGetReadCode25niT mtGetReadCode25ni;
        private mtSetReadCode25niT mtSetReadCode25ni;
        private mtSetPhotometricT mtSetPhotometric;
        private mtGetPhotometricT mtGetPhotometric;
        private mtSetAllowDuplicateValuesT mtSetAllowDuplicateValues;
        private mtGetAllowDuplicateValuesT mtGetAllowDuplicateValues;
        private mtSetDespeckleT mtSetDespeckle;
        private mtGetDespeckleT mtGetDespeckle;
        private mtGetMinSeparationT mtGetMinSeparation;
        private mtSetMinSeparationT mtSetMinSeparation;
        private mtGetBarStringQualityScoreT mtGetBarStringQualityScore;
        private mtSetExtendedCode39T mtSetExtendedCode39;
        private mtGetExtendedCode39T mtGetExtendedCode39;
        private mtSetCode39ChecksumT mtSetCode39Checksum;
        private mtGetCode39ChecksumT mtGetCode39Checksum;
        private mtSetErrorCorrectionT mtSetErrorCorrection;
        private mtGetErrorCorrectionT mtGetErrorCorrection;
        private mtSetReadNumericT mtSetReadNumeric;
        private mtGetReadNumericT mtGetReadNumeric;
        private mtSetPatternT mtSetPattern;
        private mtGetPatternT mtGetPattern;
        private mtSetPdfBppT mtSetPdfBpp;
        private mtGetPdfBppT mtGetPdfBpp;
        private mtSetPdfDpiT mtSetPdfDpi;
        private mtGetPdfDpiT mtGetPdfDpi;
        private mtSetMinSpaceBarWidthT mtSetMinSpaceBarWidth;
        private mtGetMinSpaceBarWidthT mtGetMinSpaceBarWidth;
        private mtGetReadPDF417T mtGetReadPDF417;
        private mtSetReadPDF417T mtSetReadPDF417;
        private mtGetReadMicroPDF417T mtGetReadMicroPDF417;
        private mtSetReadMicroPDF417T mtSetReadMicroPDF417;
        private mtGetMedianFilterT mtGetMedianFilter;
        private mtSetMedianFilterT mtSetMedianFilter;
        private mtSaveResultsT mtSaveResults;
        private mtLoadXMLSettingsT mtLoadXMLSettings;
        private mtExportXMLSettingsT mtExportXMLSettings;
        private mtProcessXMLT mtProcessXML;
        private mtGetCode25ChecksumT mtGetCode25Checksum;
        private mtSetCode25ChecksumT mtSetCode25Checksum;
        private mtSetEncodingT mtSetEncoding;
        private mtGetEncodingT mtGetEncoding;
        private mtSetGammaCorrectionT mtSetGammaCorrection;
        private mtGetGammaCorrectionT mtGetGammaCorrection;
        private mtSetSkewLineJumpT mtSetSkewLineJump;
        private mtGetSkewLineJumpT mtGetSkewLineJump;
        private mtGetReadDataMatrixT mtGetReadDataMatrix;
        private mtSetReadDataMatrixT mtSetReadDataMatrix;
        private mtGetReadQRCodeT mtGetReadQRCode;
        private mtSetReadQRCodeT mtSetReadQRCode;
        private mtGetReadDatabarT mtGetReadDatabar;
        private mtSetReadDatabarT mtSetReadDatabar;
        private mtGetColorProcessingLevelT mtGetColorProcessingLevel;
        private mtSetColorProcessingLevelT mtSetColorProcessingLevel;
        private mtGetDatabarOptionsT mtGetDatabarOptions;
        private mtSetDatabarOptionsT mtSetDatabarOptions;
        private mtGetPdfImageOnlyT mtGetPdfImageOnly;
        private mtSetPdfImageOnlyT mtSetPdfImageOnly;
        private mtGetReadCode93T mtGetReadCode93;
        private mtSetReadCode93T mtSetReadCode93;
        private mtGetPdfImageExtractOptionsT mtGetPdfImageExtractOptions;
        private mtSetPdfImageExtractOptionsT mtSetPdfImageExtractOptions;
        private mtGetPdfImageRasterOptionsT mtGetPdfImageRasterOptions;
        private mtSetPdfImageRasterOptionsT mtSetPdfImageRasterOptions;
        private getFunctionalityLevelT getFunctionalityLevel;
        private mtGetLastErrorT mtGetLastError;
        private mtGetLastWarningT mtGetLastWarning;
        private mtGetLastWinErrorT mtGetLastWinError;
        private mtGetUseFastScanT mtGetUseFastScan;
        private mtSetUseFastScanT mtSetUseFastScan;
        private mtGetBarcodesAtTopOfPageT mtGetBarcodesAtTopOfPage;
        private mtSetBarcodesAtTopOfPageT mtSetBarcodesAtTopOfPage;
        private mtGetFastScanLineJumpT mtGetFastScanLineJump;
        private mtSetFastScanLineJumpT mtSetFastScanLineJump;
        private mtGetMaxBarcodesPerPageT mtGetMaxBarcodesPerPage;
        private mtSetMaxBarcodesPerPageT mtSetMaxBarcodesPerPage;
        private mtGetSkewedDatamatrixT mtGetSkewedDatamatrix;
        private mtSetSkewedDatamatrixT mtSetSkewedDatamatrix;
        private mtGetTimeOutT mtGetTimeOut;
        private mtSetTimeOutT mtSetTimeOut;
        private mtGetSkewedLinearT mtGetSkewedLinear;
        private mtSetSkewedLinearT mtSetSkewedLinear;
        private mtGetReportUnreadBarcodesT mtGetReportUnreadBarcodes;
        private mtSetReportUnreadBarcodesT mtSetReportUnreadBarcodes;
        private mtGetShowCodabarStartStopT mtGetShowCodabarStartStop;
        private mtSetShowCodabarStartStopT mtSetShowCodabarStartStop;
        private mtGetProgressT mtGetProgress;
        private mtScanBarCodeInBackgroundT mtScanBarCodeInBackground;
        private mtScanBarCodeFromStringInBackgroundT mtScanBarCodeFromStringInBackground;
        private mtScanBarCodeWaitT mtScanBarCodeWait;
        private mtGetBarCodeCountT mtGetBarCodeCount;
        private mtGetBlankPageCountT mtGetBlankPageCount;
        private mtGetBlankPageT mtGetBlankPage;
        private mtScanBarCodeAbortT mtScanBarCodeAbort;
        private mtGetScanExitCodeT mtGetScanExitCode;
        private mtGetMaxThreadsT mtGetMaxThreads;
        private mtSetMaxThreadsT mtSetMaxThreads;
        private mtSetRotateBy45IfNoBarcodeT mtSetRotateBy45IfNoBarcode;
        private mtGetRotateBy45IfNoBarcodeT mtGetRotateBy45IfNoBarcode;
        private mtSetCode128DebugModeT mtSetCode128DebugMode;
        private mtGetCode128DebugModeT mtGetCode128DebugMode;
        private mtSetCode128LenientT mtSetCode128Lenient;
        private mtGetCode128LenientT mtGetCode128Lenient;
        private mtSetCode128SearchLevelT mtSetCode128SearchLevel;
        private mtGetCode128SearchLevelT mtGetCode128SearchLevel;
        private mtSetMedianFilterBiasT mtSetMedianFilterBias;
        private mtGetMedianFilterBiasT mtGetMedianFilterBias;
        private mtSetPdf417DebugT mtSetPdf417Debug;
        private mtGetPdf417DebugT mtGetPdf417Debug;
        private mtSetPdf417ChannelModeT mtSetPdf417ChannelMode;
        private mtGetPdf417ChannelModeT mtGetPdf417ChannelMode;
        private mtSetPdfLockingT mtSetPdfLocking;
        private mtGetPdfLockingT mtGetPdfLocking;
        private mtSetQrCodeReadInvertedT mtSetQrCodeReadInverted;
        private mtGetQrCodeReadInvertedT mtGetQrCodeReadInverted;
        private mtSetQuotedPrintableCharSetT mtSetQuotedPrintableCharSet;
        private mtGetQuotedPrintableCharSetT mtGetQuotedPrintableCharSet;
        private mtSetWeightLongerBarcodesT mtSetWeightLongerBarcodes;
        private mtGetWeightLongerBarcodesT mtGetWeightLongerBarcodes;
        private mtSetCode39MaxRatioPcntT mtSetCode39MaxRatioPcnt;
        private mtGetCode39MaxRatioPcntT mtGetCode39MaxRatioPcnt;
        private mtSetDataMatrixFinderGapToleranceT mtSetDataMatrixFinderGapTolerance;
        private mtGetDataMatrixFinderGapToleranceT mtGetDataMatrixFinderGapTolerance;
        private mtSetDataMatrixAutoUTF8T mtSetDataMatrixAutoUTF8;
        private mtGetDataMatrixAutoUTF8T mtGetDataMatrixAutoUTF8;
        private mtSetPatchCodeMinOccurrenceT mtSetPatchCodeMinOccurrence;
        private mtGetPatchCodeMinOccurrenceT mtGetPatchCodeMinOccurrence;
        private mtSetPdf417AutoUTF8T mtSetPdf417AutoUTF8;
        private mtGetPdf417AutoUTF8T mtGetPdf417AutoUTF8;
        private mtSetPdf417MacroEscapeBackslashT mtSetPdf417MacroEscapeBackslash;
        private mtGetPdf417MacroEscapeBackslashT mtGetPdf417MacroEscapeBackslash;
        private mtSetQRCodeAutoUTF8T mtSetQRCodeAutoUTF8;
        private mtGetQRCodeAutoUTF8T mtGetQRCodeAutoUTF8;
        private mtSetQRCodeAutoMedianFilterT mtSetQRCodeAutoMedianFilter;
        private mtGetQRCodeAutoMedianFilterT mtGetQRCodeAutoMedianFilter;
        private mtSetUseOldCode128AlgorithmT mtSetUseOldCode128Algorithm;
        private mtGetUseOldCode128AlgorithmT mtGetUseOldCode128Algorithm;
        private mtSetUseRunCacheT mtSetUseRunCache;
        private mtGetUseRunCacheT mtGetUseRunCache;
        private mtSetDeskewModeT mtSetDeskewMode;
        private mtGetDeskewModeT mtGetDeskewMode;
        private mtSetPdfPasswordT mtSetPdfPassword;
        private mtGetPdfPasswordT mtGetPdfPassword;
        private mtGetDataMatrixSearchLevelT mtGetDataMatrixSearchLevel;
        private mtSetDataMatrixSearchLevelT mtSetDataMatrixSearchLevel;
        private mtGetDataMatrixRectangleSupportT mtGetDataMatrixRectangleSupport;
        private mtSetDataMatrixRectangleSupportT mtSetDataMatrixRectangleSupport;
        private mtGetReadEAN13SupplementalT mtGetReadEAN13Supplemental;
        private mtSetReadEAN13SupplementalT mtSetReadEAN13Supplemental;
        private mtGetIndexBlankPagesT mtGetIndexBlankPages;
        private mtSetIndexBlankPagesT mtSetIndexBlankPages;
        private mtGetBlankPageStandardDeviationT mtGetBlankPageStandardDeviation;
        private mtSetBlankPageStandardDeviationT mtSetBlankPageStandardDeviation;
        private mtGetBlankPageMarginT mtGetBlankPageMargin;
        private mtSetBlankPageMarginT mtSetBlankPageMargin;





        private void loadDLLFuncs()
        {
            mtGetLineJumpA = (mtGetLineJumpAT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetLineJumpA"), typeof(mtGetLineJumpAT));
            mtCreateBarcodeInstance = (mtCreateBarcodeInstanceT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtCreateBarcodeInstance"), typeof(mtCreateBarcodeInstanceT));
            mtDestroyBarcodeInstance = (mtDestroyBarcodeInstanceT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtDestroyBarcodeInstance"), typeof(mtDestroyBarcodeInstanceT));
            mtGetBitmapResolution = (mtGetBitmapResolutionT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBitmapResolution"), typeof(mtGetBitmapResolutionT));
            mtSetBitmapResolution = (mtSetBitmapResolutionT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetBitmapResolution"), typeof(mtSetBitmapResolutionT));
            mtGetDebugTraceFile = (mtGetDebugTraceFileT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetDebugTraceFile"), typeof(mtGetDebugTraceFileT));
            mtSetDebugTraceFile = (mtSetDebugTraceFileT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetDebugTraceFile"), typeof(mtSetDebugTraceFileT));
            mtSetLicenseKey = (mtSetLicenseKeyT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetLicenseKey"), typeof(mtSetLicenseKeyT));
            mtGetLicenseKey = (mtGetLicenseKeyT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetLicenseKey"), typeof(mtGetLicenseKeyT));
            mtSetLineJumpA = (mtSetLineJumpAT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetLineJumpA"), typeof(mtSetLineJumpAT));
            mtGetLowerRatio = (mtGetLowerRatioT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetLowerRatio"), typeof(mtGetLowerRatioT));
            mtSetLowerRatio = (mtSetLowerRatioT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetLowerRatio"), typeof(mtSetLowerRatioT));
            mtGetMinOccurrence = (mtGetMinOccurrenceT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetMinOccurrence"), typeof(mtGetMinOccurrenceT));
            mtSetMinOccurrence = (mtSetMinOccurrenceT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetMinOccurrence"), typeof(mtSetMinOccurrenceT));
            mtGetMultipleRead = (mtGetMultipleReadT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetMultipleRead"), typeof(mtGetMultipleReadT));
            mtSetMultipleRead = (mtSetMultipleReadT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetMultipleRead"), typeof(mtSetMultipleReadT));
            mtGetNoiseReduction = (mtGetNoiseReductionT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetNoiseReduction"), typeof(mtGetNoiseReductionT));
            mtSetNoiseReduction = (mtSetNoiseReductionT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetNoiseReduction"), typeof(mtSetNoiseReductionT));
            mtGetPageNo = (mtGetPageNoT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPageNo"), typeof(mtGetPageNoT));
            mtSetPageNo = (mtSetPageNoT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPageNo"), typeof(mtSetPageNoT));
            mtGetPrefOccurrence = (mtGetPrefOccurrenceT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPrefOccurrence"), typeof(mtGetPrefOccurrenceT));
            mtSetPrefOccurrence = (mtSetPrefOccurrenceT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPrefOccurrence"), typeof(mtSetPrefOccurrenceT));
            mtGetQuietZoneSize = (mtGetQuietZoneSizeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetQuietZoneSize"), typeof(mtGetQuietZoneSizeT));
            mtSetQuietZoneSize = (mtSetQuietZoneSizeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetQuietZoneSize"), typeof(mtSetQuietZoneSizeT));
            mtGetRotation = (mtGetRotationT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetRotation"), typeof(mtGetRotationT));
            mtSetRotation = (mtSetRotationT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetRotation"), typeof(mtSetRotationT));
            mtGetUpperRatio = (mtGetUpperRatioT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetUpperRatio"), typeof(mtGetUpperRatioT));
            mtSetUpperRatio = (mtSetUpperRatioT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetUpperRatio"), typeof(mtSetUpperRatioT));
            mtGetBarString = (mtGetBarStringT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBarString"), typeof(mtGetBarStringT));
            mtGetBarStringA = (mtGetBarStringAT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBarStringA"), typeof(mtGetBarStringAT));
            mtGetRawBarString = (mtGetRawBarStringT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetRawBarString"), typeof(mtGetRawBarStringT));
            mtScanBarCode = (mtScanBarCodeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtScanBarCode"), typeof(mtScanBarCodeT));
            mtSetPDFTool = (mtSetPDFToolT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPDFTool"), typeof(mtSetPDFToolT));
            mtGetPDFTool = (mtGetPDFToolT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPDFTool"), typeof(mtGetPDFToolT));
            mtScanBarCodeFromDIB = (mtScanBarCodeFromDIBT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtScanBarCodeFromDIB"), typeof(mtScanBarCodeFromDIBT));
            mtScanBarCodeFromBitmap = (mtScanBarCodeFromBitmapT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtScanBarCodeFromBitmap"), typeof(mtScanBarCodeFromBitmapT));
            mtScanBarCodeFromString = (mtScanBarCodeFromStringT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtScanBarCodeFromString"), typeof(mtScanBarCodeFromStringT));
            mtScanBarCodeFromBitmapA = (mtScanBarCodeFromBitmapAT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtScanBarCodeFromBitmapA"), typeof(mtScanBarCodeFromBitmapAT));
            mtGetReadCode39 = (mtGetReadCode39T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadCode39"), typeof(mtGetReadCode39T));
            mtSetReadCode39 = (mtSetReadCode39T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadCode39"), typeof(mtSetReadCode39T));
            mtGetReadEAN13 = (mtGetReadEAN13T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadEAN13"), typeof(mtGetReadEAN13T));
            mtSetReadEAN13 = (mtSetReadEAN13T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadEAN13"), typeof(mtSetReadEAN13T));
            mtGetReadEAN8 = (mtGetReadEAN8T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadEAN8"), typeof(mtGetReadEAN8T));
            mtSetReadEAN8 = (mtSetReadEAN8T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadEAN8"), typeof(mtSetReadEAN8T));
            mtGetReadUPCA = (mtGetReadUPCAT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadUPCA"), typeof(mtGetReadUPCAT));
            mtSetReadUPCA = (mtSetReadUPCAT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadUPCA"), typeof(mtSetReadUPCAT));
            mtGetReadUPCE = (mtGetReadUPCET)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadUPCE"), typeof(mtGetReadUPCET));
            mtSetReadUPCE = (mtSetReadUPCET)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadUPCE"), typeof(mtSetReadUPCET));
            mtGetShowCheckDigit = (mtGetShowCheckDigitT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetShowCheckDigit"), typeof(mtGetShowCheckDigitT));
            mtSetShowCheckDigit = (mtSetShowCheckDigitT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetShowCheckDigit"), typeof(mtSetShowCheckDigitT));
            mtGetBarStringType = (mtGetBarStringTypeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBarStringType"), typeof(mtGetBarStringTypeT));
            mtGetReadCode128 = (mtGetReadCode128T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadCode128"), typeof(mtGetReadCode128T));
            mtSetReadCode128 = (mtSetReadCode128T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadCode128"), typeof(mtSetReadCode128T));
            mtGetReadShortCode128 = (mtGetReadShortCode128T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadShortCode128"), typeof(mtGetReadShortCode128T));
            mtSetReadShortCode128 = (mtSetReadShortCode128T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadShortCode128"), typeof(mtSetReadShortCode128T));
            mtGetCode39NeedStartStop = (mtGetCode39NeedStartStopT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetCode39NeedStartStop"), typeof(mtGetCode39NeedStartStopT));
            mtSetCode39NeedStartStop = (mtSetCode39NeedStartStopT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetCode39NeedStartStop"), typeof(mtSetCode39NeedStartStopT));
            mtGetColorThreshold = (mtGetColorThresholdT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetColorThreshold"), typeof(mtGetColorThresholdT));
            mtSetColorThreshold = (mtSetColorThresholdT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetColorThreshold"), typeof(mtSetColorThresholdT));
            mtGetBarStringPage = (mtGetBarStringPageT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBarStringPage"), typeof(mtGetBarStringPageT));
            mtGetBarStringTlx = (mtGetBarStringTlxT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBarStringTlx"), typeof(mtGetBarStringTlxT));
            mtGetBarStringTly = (mtGetBarStringTlyT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBarStringTly"), typeof(mtGetBarStringTlyT));
            mtGetBarStringBrx = (mtGetBarStringBrxT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBarStringBrx"), typeof(mtGetBarStringBrxT));
            mtGetBarStringBry = (mtGetBarStringBryT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBarStringBry"), typeof(mtGetBarStringBryT));
            mtGetReadCode25 = (mtGetReadCode25T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadCode25"), typeof(mtGetReadCode25T));
            mtSetReadCode25 = (mtSetReadCode25T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadCode25"), typeof(mtSetReadCode25T));
            mtSetScanRectA = (mtSetScanRectAT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetScanRectA"), typeof(mtSetScanRectAT));
            mtGetReadPatchCodes = (mtGetReadPatchCodesT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadPatchCodes"), typeof(mtGetReadPatchCodesT));
            mtSetReadPatchCodes = (mtSetReadPatchCodesT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadPatchCodes"), typeof(mtSetReadPatchCodesT));
            mtGetUseOverSampling = (mtGetUseOverSamplingT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetUseOverSampling"), typeof(mtGetUseOverSamplingT));
            mtSetUseOverSampling = (mtSetUseOverSamplingT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetUseOverSampling"), typeof(mtSetUseOverSamplingT));
            mtGetScanDirection = (mtGetScanDirectionT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetScanDirection"), typeof(mtGetScanDirectionT));
            mtSetScanDirection = (mtSetScanDirectionT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetScanDirection"), typeof(mtSetScanDirectionT));
            mtGetMinLength = (mtGetMinLengthT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetMinLength"), typeof(mtGetMinLengthT));
            mtSetMinLength = (mtSetMinLengthT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetMinLength"), typeof(mtSetMinLengthT));
            mtGetMin2DLength = (mtGetMin2DLengthT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetMin2DLength"), typeof(mtGetMin2DLengthT));
            mtSetMin2DLength = (mtSetMin2DLengthT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetMin2DLength"), typeof(mtSetMin2DLengthT));
            mtGetShortCode128MinLength = (mtGetShortCode128MinLengthT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetShortCode128MinLength"), typeof(mtGetShortCode128MinLengthT));
            mtSetShortCode128MinLength = (mtSetShortCode128MinLengthT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetShortCode128MinLength"), typeof(mtSetShortCode128MinLengthT));
            mtGetMaxLength = (mtGetMaxLengthT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetMaxLength"), typeof(mtGetMaxLengthT));
            mtSetMaxLength = (mtSetMaxLengthT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetMaxLength"), typeof(mtSetMaxLengthT));
            mtSetTifSplitPath = (mtSetTifSplitPathT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetTifSplitPath"), typeof(mtSetTifSplitPathT));
            mtGetTifSplitPath = (mtGetTifSplitPathT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetTifSplitPath"), typeof(mtGetTifSplitPathT));
            mtSetTifSplitValues = (mtSetTifSplitValuesT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetTifSplitValues"), typeof(mtSetTifSplitValuesT));
            mtGetTifSplitValues = (mtGetTifSplitValuesT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetTifSplitValues"), typeof(mtGetTifSplitValuesT));
            mtGetTifSplitMode = (mtGetTifSplitModeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetTifSplitMode"), typeof(mtGetTifSplitModeT));
            mtSetTifSplitMode = (mtSetTifSplitModeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetTifSplitMode"), typeof(mtSetTifSplitModeT));
            mtGetReadCodabar = (mtGetReadCodabarT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadCodabar"), typeof(mtGetReadCodabarT));
            mtSetReadCodabar = (mtSetReadCodabarT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadCodabar"), typeof(mtSetReadCodabarT));
            mtGetBarStringDirection = (mtGetBarStringDirectionT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBarStringDirection"), typeof(mtGetBarStringDirectionT));
            mtGetConvertUPCEToEAN13 = (mtGetConvertUPCEToEAN13T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetConvertUPCEToEAN13"), typeof(mtGetConvertUPCEToEAN13T));
            mtSetConvertUPCEToEAN13 = (mtSetConvertUPCEToEAN13T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetConvertUPCEToEAN13"), typeof(mtSetConvertUPCEToEAN13T));
            mtScanBarCodeFromBitmapDump = (mtScanBarCodeFromBitmapDumpT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtScanBarCodeFromBitmapDump"), typeof(mtScanBarCodeFromBitmapDumpT));
            mtGetSkewTolerance = (mtGetSkewToleranceT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetSkewTolerance"), typeof(mtGetSkewToleranceT));
            mtSetSkewTolerance = (mtSetSkewToleranceT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetSkewTolerance"), typeof(mtSetSkewToleranceT));
            mtGetReadCode25ni = (mtGetReadCode25niT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadCode25ni"), typeof(mtGetReadCode25niT));
            mtSetReadCode25ni = (mtSetReadCode25niT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadCode25ni"), typeof(mtSetReadCode25niT));
            mtSetPhotometric = (mtSetPhotometricT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPhotometric"), typeof(mtSetPhotometricT));
            mtGetPhotometric = (mtGetPhotometricT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPhotometric"), typeof(mtGetPhotometricT));
            mtSetAllowDuplicateValues = (mtSetAllowDuplicateValuesT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetAllowDuplicateValues"), typeof(mtSetAllowDuplicateValuesT));
            mtGetAllowDuplicateValues = (mtGetAllowDuplicateValuesT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetAllowDuplicateValues"), typeof(mtGetAllowDuplicateValuesT));
            mtSetDespeckle = (mtSetDespeckleT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetDespeckle"), typeof(mtSetDespeckleT));
            mtGetDespeckle = (mtGetDespeckleT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetDespeckle"), typeof(mtGetDespeckleT));
            mtGetMinSeparation = (mtGetMinSeparationT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetMinSeparation"), typeof(mtGetMinSeparationT));
            mtSetMinSeparation = (mtSetMinSeparationT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetMinSeparation"), typeof(mtSetMinSeparationT));
            mtGetBarStringQualityScore = (mtGetBarStringQualityScoreT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBarStringQualityScore"), typeof(mtGetBarStringQualityScoreT));
            mtSetExtendedCode39 = (mtSetExtendedCode39T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetExtendedCode39"), typeof(mtSetExtendedCode39T));
            mtGetExtendedCode39 = (mtGetExtendedCode39T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetExtendedCode39"), typeof(mtGetExtendedCode39T));
            mtSetCode39Checksum = (mtSetCode39ChecksumT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetCode39Checksum"), typeof(mtSetCode39ChecksumT));
            mtGetCode39Checksum = (mtGetCode39ChecksumT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetCode39Checksum"), typeof(mtGetCode39ChecksumT));
            mtSetErrorCorrection = (mtSetErrorCorrectionT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetErrorCorrection"), typeof(mtSetErrorCorrectionT));
            mtGetErrorCorrection = (mtGetErrorCorrectionT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetErrorCorrection"), typeof(mtGetErrorCorrectionT));
            mtSetReadNumeric = (mtSetReadNumericT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadNumeric"), typeof(mtSetReadNumericT));
            mtGetReadNumeric = (mtGetReadNumericT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadNumeric"), typeof(mtGetReadNumericT));
            mtSetPattern = (mtSetPatternT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPattern"), typeof(mtSetPatternT));
            mtGetPattern = (mtGetPatternT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPattern"), typeof(mtGetPatternT));
            mtSetPdfBpp = (mtSetPdfBppT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPdfBpp"), typeof(mtSetPdfBppT));
            mtGetPdfBpp = (mtGetPdfBppT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPdfBpp"), typeof(mtGetPdfBppT));
            mtSetPdfDpi = (mtSetPdfDpiT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPdfDpi"), typeof(mtSetPdfDpiT));
            mtGetPdfDpi = (mtGetPdfDpiT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPdfDpi"), typeof(mtGetPdfDpiT));
            mtSetMinSpaceBarWidth = (mtSetMinSpaceBarWidthT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetMinSpaceBarWidth"), typeof(mtSetMinSpaceBarWidthT));
            mtGetMinSpaceBarWidth = (mtGetMinSpaceBarWidthT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetMinSpaceBarWidth"), typeof(mtGetMinSpaceBarWidthT));
            mtGetReadPDF417 = (mtGetReadPDF417T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadPDF417"), typeof(mtGetReadPDF417T));
            mtSetReadPDF417 = (mtSetReadPDF417T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadPDF417"), typeof(mtSetReadPDF417T));
            mtGetReadMicroPDF417 = (mtGetReadMicroPDF417T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadMicroPDF417"), typeof(mtGetReadMicroPDF417T));
            mtSetReadMicroPDF417 = (mtSetReadMicroPDF417T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadMicroPDF417"), typeof(mtSetReadMicroPDF417T));
            mtGetMedianFilter = (mtGetMedianFilterT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetMedianFilter"), typeof(mtGetMedianFilterT));
            mtSetMedianFilter = (mtSetMedianFilterT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetMedianFilter"), typeof(mtSetMedianFilterT));
            mtSaveResults = (mtSaveResultsT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSaveResults"), typeof(mtSaveResultsT));
            mtLoadXMLSettings = (mtLoadXMLSettingsT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtLoadXMLSettings"), typeof(mtLoadXMLSettingsT));
            mtExportXMLSettings = (mtExportXMLSettingsT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtExportXMLSettings"), typeof(mtExportXMLSettingsT));
            mtProcessXML = (mtProcessXMLT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtProcessXML"), typeof(mtProcessXMLT));
            mtGetCode25Checksum = (mtGetCode25ChecksumT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetCode25Checksum"), typeof(mtGetCode25ChecksumT));
            mtSetCode25Checksum = (mtSetCode25ChecksumT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetCode25Checksum"), typeof(mtSetCode25ChecksumT));
            mtSetEncoding = (mtSetEncodingT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetEncoding"), typeof(mtSetEncodingT));
            mtGetEncoding = (mtGetEncodingT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetEncoding"), typeof(mtGetEncodingT));
            mtSetGammaCorrection = (mtSetGammaCorrectionT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetGammaCorrection"), typeof(mtSetGammaCorrectionT));
            mtGetGammaCorrection = (mtGetGammaCorrectionT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetGammaCorrection"), typeof(mtGetGammaCorrectionT));
            mtSetSkewLineJump = (mtSetSkewLineJumpT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetSkewLineJump"), typeof(mtSetSkewLineJumpT));
            mtGetSkewLineJump = (mtGetSkewLineJumpT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetSkewLineJump"), typeof(mtGetSkewLineJumpT));
            mtGetReadDataMatrix = (mtGetReadDataMatrixT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadDataMatrix"), typeof(mtGetReadDataMatrixT));
            mtSetReadDataMatrix = (mtSetReadDataMatrixT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadDataMatrix"), typeof(mtSetReadDataMatrixT));
            mtGetReadQRCode = (mtGetReadQRCodeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadQRCode"), typeof(mtGetReadQRCodeT));
            mtSetReadQRCode = (mtSetReadQRCodeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadQRCode"), typeof(mtSetReadQRCodeT));
            mtGetReadDatabar = (mtGetReadDatabarT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadDatabar"), typeof(mtGetReadDatabarT));
            mtSetReadDatabar = (mtSetReadDatabarT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadDatabar"), typeof(mtSetReadDatabarT));
            mtGetColorProcessingLevel = (mtGetColorProcessingLevelT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetColorProcessingLevel"), typeof(mtGetColorProcessingLevelT));
            mtSetColorProcessingLevel = (mtSetColorProcessingLevelT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetColorProcessingLevel"), typeof(mtSetColorProcessingLevelT));
            mtGetDatabarOptions = (mtGetDatabarOptionsT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetDatabarOptions"), typeof(mtGetDatabarOptionsT));
            mtSetDatabarOptions = (mtSetDatabarOptionsT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetDatabarOptions"), typeof(mtSetDatabarOptionsT));
            mtGetPdfImageOnly = (mtGetPdfImageOnlyT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPdfImageOnly"), typeof(mtGetPdfImageOnlyT));
            mtSetPdfImageOnly = (mtSetPdfImageOnlyT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPdfImageOnly"), typeof(mtSetPdfImageOnlyT));
            mtGetReadCode93 = (mtGetReadCode93T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadCode93"), typeof(mtGetReadCode93T));
            mtSetReadCode93 = (mtSetReadCode93T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadCode93"), typeof(mtSetReadCode93T));
            mtGetPdfImageExtractOptions = (mtGetPdfImageExtractOptionsT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPdfImageExtractOptions"), typeof(mtGetPdfImageExtractOptionsT));
            mtSetPdfImageExtractOptions = (mtSetPdfImageExtractOptionsT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPdfImageExtractOptions"), typeof(mtSetPdfImageExtractOptionsT));
            mtGetPdfImageRasterOptions = (mtGetPdfImageRasterOptionsT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPdfImageRasterOptions"), typeof(mtGetPdfImageRasterOptionsT));
            mtSetPdfImageRasterOptions = (mtSetPdfImageRasterOptionsT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPdfImageRasterOptions"), typeof(mtSetPdfImageRasterOptionsT));
            getFunctionalityLevel = (getFunctionalityLevelT)Marshal.GetDelegateForFunctionPointer(loadFunc("getFunctionalityLevel"), typeof(getFunctionalityLevelT));
            mtGetLastError = (mtGetLastErrorT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetLastError"), typeof(mtGetLastErrorT));
            mtGetLastWarning = (mtGetLastWarningT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetLastWarning"), typeof(mtGetLastWarningT));
            mtGetLastWinError = (mtGetLastWinErrorT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetLastWinError"), typeof(mtGetLastWinErrorT));
            mtGetUseFastScan = (mtGetUseFastScanT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetUseFastScan"), typeof(mtGetUseFastScanT));
            mtSetUseFastScan = (mtSetUseFastScanT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetUseFastScan"), typeof(mtSetUseFastScanT));
            mtGetBarcodesAtTopOfPage = (mtGetBarcodesAtTopOfPageT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBarcodesAtTopOfPage"), typeof(mtGetBarcodesAtTopOfPageT));
            mtSetBarcodesAtTopOfPage = (mtSetBarcodesAtTopOfPageT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetBarcodesAtTopOfPage"), typeof(mtSetBarcodesAtTopOfPageT));
            mtGetFastScanLineJump = (mtGetFastScanLineJumpT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetFastScanLineJump"), typeof(mtGetFastScanLineJumpT));
            mtSetFastScanLineJump = (mtSetFastScanLineJumpT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetFastScanLineJump"), typeof(mtSetFastScanLineJumpT));
            mtGetMaxBarcodesPerPage = (mtGetMaxBarcodesPerPageT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetMaxBarcodesPerPage"), typeof(mtGetMaxBarcodesPerPageT));
            mtSetMaxBarcodesPerPage = (mtSetMaxBarcodesPerPageT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetMaxBarcodesPerPage"), typeof(mtSetMaxBarcodesPerPageT));
            mtGetSkewedDatamatrix = (mtGetSkewedDatamatrixT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetSkewedDatamatrix"), typeof(mtGetSkewedDatamatrixT));
            mtSetSkewedDatamatrix = (mtSetSkewedDatamatrixT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetSkewedDatamatrix"), typeof(mtSetSkewedDatamatrixT));
            mtGetTimeOut = (mtGetTimeOutT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetTimeOut"), typeof(mtGetTimeOutT));
            mtSetTimeOut = (mtSetTimeOutT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetTimeOut"), typeof(mtSetTimeOutT));
            mtGetSkewedLinear = (mtGetSkewedLinearT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetSkewedLinear"), typeof(mtGetSkewedLinearT));
            mtSetSkewedLinear = (mtSetSkewedLinearT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetSkewedLinear"), typeof(mtSetSkewedLinearT));
            mtGetReportUnreadBarcodes = (mtGetReportUnreadBarcodesT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReportUnreadBarcodes"), typeof(mtGetReportUnreadBarcodesT));
            mtSetReportUnreadBarcodes = (mtSetReportUnreadBarcodesT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReportUnreadBarcodes"), typeof(mtSetReportUnreadBarcodesT));
            mtGetShowCodabarStartStop = (mtGetShowCodabarStartStopT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetShowCodabarStartStop"), typeof(mtGetShowCodabarStartStopT));
            mtSetShowCodabarStartStop = (mtSetShowCodabarStartStopT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetShowCodabarStartStop"), typeof(mtSetShowCodabarStartStopT));
            mtGetProgress = (mtGetProgressT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetProgress"), typeof(mtGetProgressT));
            mtScanBarCodeInBackground = (mtScanBarCodeInBackgroundT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtScanBarCodeInBackground"), typeof(mtScanBarCodeInBackgroundT));
            mtScanBarCodeFromStringInBackground = (mtScanBarCodeFromStringInBackgroundT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtScanBarCodeFromStringInBackground"), typeof(mtScanBarCodeFromStringInBackgroundT));
            mtScanBarCodeWait = (mtScanBarCodeWaitT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtScanBarCodeWait"), typeof(mtScanBarCodeWaitT));
            mtGetBarCodeCount = (mtGetBarCodeCountT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBarCodeCount"), typeof(mtGetBarCodeCountT));
            mtGetBlankPageCount = (mtGetBlankPageCountT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBlankPageCount"), typeof(mtGetBlankPageCountT));
            mtGetBlankPage = (mtGetBlankPageT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBlankPage"), typeof(mtGetBlankPageT));
            mtScanBarCodeAbort = (mtScanBarCodeAbortT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtScanBarCodeAbort"), typeof(mtScanBarCodeAbortT));
            mtGetScanExitCode = (mtGetScanExitCodeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetScanExitCode"), typeof(mtGetScanExitCodeT));
            mtGetMaxThreads = (mtGetMaxThreadsT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetMaxThreads"), typeof(mtGetMaxThreadsT));
            mtSetMaxThreads = (mtSetMaxThreadsT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetMaxThreads"), typeof(mtSetMaxThreadsT));
            mtGetRotateBy45IfNoBarcode = (mtGetRotateBy45IfNoBarcodeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetRotateBy45IfNoBarcode"), typeof(mtGetRotateBy45IfNoBarcodeT));
            mtGetCode128DebugMode = (mtGetCode128DebugModeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetCode128DebugMode"), typeof(mtGetCode128DebugModeT));
            mtGetCode128Lenient = (mtGetCode128LenientT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetCode128Lenient"), typeof(mtGetCode128LenientT));
            mtGetCode128SearchLevel = (mtGetCode128SearchLevelT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetCode128SearchLevel"), typeof(mtGetCode128SearchLevelT));
            mtGetMedianFilterBias = (mtGetMedianFilterBiasT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetMedianFilterBias"), typeof(mtGetMedianFilterBiasT));
            mtGetPdf417Debug = (mtGetPdf417DebugT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPdf417Debug"), typeof(mtGetPdf417DebugT));
            mtGetPdf417ChannelMode = (mtGetPdf417ChannelModeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPdf417ChannelMode"), typeof(mtGetPdf417ChannelModeT));
            mtGetPdfLocking = (mtGetPdfLockingT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPdfLocking"), typeof(mtGetPdfLockingT));
            mtGetQrCodeReadInverted = (mtGetQrCodeReadInvertedT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetQrCodeReadInverted"), typeof(mtGetQrCodeReadInvertedT));
            mtGetQuotedPrintableCharSet = (mtGetQuotedPrintableCharSetT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetQuotedPrintableCharSet"), typeof(mtGetQuotedPrintableCharSetT));
            mtGetWeightLongerBarcodes = (mtGetWeightLongerBarcodesT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetWeightLongerBarcodes"), typeof(mtGetWeightLongerBarcodesT));
            mtGetCode39MaxRatioPcnt = (mtGetCode39MaxRatioPcntT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetCode39MaxRatioPcnt"), typeof(mtGetCode39MaxRatioPcntT));
            mtGetDataMatrixFinderGapTolerance = (mtGetDataMatrixFinderGapToleranceT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetDataMatrixFinderGapTolerance"), typeof(mtGetDataMatrixFinderGapToleranceT));
            mtGetDataMatrixAutoUTF8 = (mtGetDataMatrixAutoUTF8T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetDataMatrixAutoUTF8"), typeof(mtGetDataMatrixAutoUTF8T));
            mtGetPatchCodeMinOccurrence = (mtGetPatchCodeMinOccurrenceT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPatchCodeMinOccurrence"), typeof(mtGetPatchCodeMinOccurrenceT));
            mtGetPdf417AutoUTF8 = (mtGetPdf417AutoUTF8T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPdf417AutoUTF8"), typeof(mtGetPdf417AutoUTF8T));
            mtGetPdf417MacroEscapeBackslash = (mtGetPdf417MacroEscapeBackslashT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPdf417MacroEscapeBackslash"), typeof(mtGetPdf417MacroEscapeBackslashT));
            mtGetQRCodeAutoUTF8 = (mtGetQRCodeAutoUTF8T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetQRCodeAutoUTF8"), typeof(mtGetQRCodeAutoUTF8T));
            mtGetQRCodeAutoMedianFilter = (mtGetQRCodeAutoMedianFilterT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetQRCodeAutoMedianFilter"), typeof(mtGetQRCodeAutoMedianFilterT));
            mtGetUseOldCode128Algorithm = (mtGetUseOldCode128AlgorithmT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetUseOldCode128Algorithm"), typeof(mtGetUseOldCode128AlgorithmT));
            mtGetUseRunCache = (mtGetUseRunCacheT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetUseRunCache"), typeof(mtGetUseRunCacheT));
            mtSetRotateBy45IfNoBarcode = (mtSetRotateBy45IfNoBarcodeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetRotateBy45IfNoBarcode"), typeof(mtSetRotateBy45IfNoBarcodeT));
            mtSetCode128DebugMode = (mtSetCode128DebugModeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetCode128DebugMode"), typeof(mtSetCode128DebugModeT));
            mtSetCode128Lenient = (mtSetCode128LenientT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetCode128Lenient"), typeof(mtSetCode128LenientT));
            mtSetCode128SearchLevel = (mtSetCode128SearchLevelT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetCode128SearchLevel"), typeof(mtSetCode128SearchLevelT));
            mtSetMedianFilterBias = (mtSetMedianFilterBiasT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetMedianFilterBias"), typeof(mtSetMedianFilterBiasT));
            mtSetPdf417Debug = (mtSetPdf417DebugT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPdf417Debug"), typeof(mtSetPdf417DebugT));
            mtSetPdf417ChannelMode = (mtSetPdf417ChannelModeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPdf417ChannelMode"), typeof(mtSetPdf417ChannelModeT));
            mtSetPdfLocking = (mtSetPdfLockingT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPdfLocking"), typeof(mtSetPdfLockingT));
            mtSetQrCodeReadInverted = (mtSetQrCodeReadInvertedT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetQrCodeReadInverted"), typeof(mtSetQrCodeReadInvertedT));
            mtSetQuotedPrintableCharSet = (mtSetQuotedPrintableCharSetT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetQuotedPrintableCharSet"), typeof(mtSetQuotedPrintableCharSetT));
            mtSetWeightLongerBarcodes = (mtSetWeightLongerBarcodesT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetWeightLongerBarcodes"), typeof(mtSetWeightLongerBarcodesT));
            mtSetCode39MaxRatioPcnt = (mtSetCode39MaxRatioPcntT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetCode39MaxRatioPcnt"), typeof(mtSetCode39MaxRatioPcntT));
            mtSetDataMatrixFinderGapTolerance = (mtSetDataMatrixFinderGapToleranceT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetDataMatrixFinderGapTolerance"), typeof(mtSetDataMatrixFinderGapToleranceT));
            mtSetDataMatrixAutoUTF8 = (mtSetDataMatrixAutoUTF8T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetDataMatrixAutoUTF8"), typeof(mtSetDataMatrixAutoUTF8T));
            mtSetPatchCodeMinOccurrence = (mtSetPatchCodeMinOccurrenceT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPatchCodeMinOccurrence"), typeof(mtSetPatchCodeMinOccurrenceT));
            mtSetPdf417AutoUTF8 = (mtSetPdf417AutoUTF8T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPdf417AutoUTF8"), typeof(mtSetPdf417AutoUTF8T));
            mtSetPdf417MacroEscapeBackslash = (mtSetPdf417MacroEscapeBackslashT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPdf417MacroEscapeBackslash"), typeof(mtSetPdf417MacroEscapeBackslashT));
            mtSetQRCodeAutoUTF8 = (mtSetQRCodeAutoUTF8T)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetQRCodeAutoUTF8"), typeof(mtSetQRCodeAutoUTF8T));
            mtSetQRCodeAutoMedianFilter = (mtSetQRCodeAutoMedianFilterT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetQRCodeAutoMedianFilter"), typeof(mtSetQRCodeAutoMedianFilterT));
            mtSetUseOldCode128Algorithm = (mtSetUseOldCode128AlgorithmT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetUseOldCode128Algorithm"), typeof(mtSetUseOldCode128AlgorithmT));
            mtSetUseRunCache = (mtSetUseRunCacheT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetUseRunCache"), typeof(mtSetUseRunCacheT));
            mtSetDeskewMode = (mtSetDeskewModeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetDeskewMode"), typeof(mtSetDeskewModeT));
            mtGetDeskewMode = (mtGetDeskewModeT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetDeskewMode"), typeof(mtGetDeskewModeT));
            mtSetPdfPassword = (mtSetPdfPasswordT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetPdfPassword"), typeof(mtSetPdfPasswordT));
            mtGetPdfPassword = (mtGetPdfPasswordT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetPdfPassword"), typeof(mtGetPdfPasswordT));
            mtSetDataMatrixSearchLevel = (mtSetDataMatrixSearchLevelT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetDataMatrixSearchLevel"), typeof(mtSetDataMatrixSearchLevelT));
            mtGetDataMatrixSearchLevel = (mtGetDataMatrixSearchLevelT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetDataMatrixSearchLevel"), typeof(mtGetDataMatrixSearchLevelT));
            mtSetDataMatrixRectangleSupport = (mtSetDataMatrixRectangleSupportT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetDataMatrixRectangleSupport"), typeof(mtSetDataMatrixRectangleSupportT));
            mtGetDataMatrixRectangleSupport = (mtGetDataMatrixRectangleSupportT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetDataMatrixRectangleSupport"), typeof(mtGetDataMatrixRectangleSupportT));
            mtSetReadEAN13Supplemental = (mtSetReadEAN13SupplementalT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetReadEAN13Supplemental"), typeof(mtSetReadEAN13SupplementalT));
            mtGetReadEAN13Supplemental = (mtGetReadEAN13SupplementalT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetReadEAN13Supplemental"), typeof(mtGetReadEAN13SupplementalT));
            mtSetIndexBlankPages = (mtSetIndexBlankPagesT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetIndexBlankPages"), typeof(mtSetIndexBlankPagesT));
            mtGetIndexBlankPages = (mtGetIndexBlankPagesT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetIndexBlankPages"), typeof(mtGetIndexBlankPagesT));
            mtSetBlankPageMargin = (mtSetBlankPageMarginT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetBlankPageMargin"), typeof(mtSetBlankPageMarginT));
            mtGetBlankPageMargin = (mtGetBlankPageMarginT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBlankPageMargin"), typeof(mtGetBlankPageMarginT));
            mtSetBlankPageStandardDeviation = (mtSetBlankPageStandardDeviationT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtSetBlankPageStandardDeviation"), typeof(mtSetBlankPageStandardDeviationT));
            mtGetBlankPageStandardDeviation = (mtGetBlankPageStandardDeviationT)Marshal.GetDelegateForFunctionPointer(loadFunc("mtGetBlankPageStandardDeviation"), typeof(mtGetBlankPageStandardDeviationT));
        }

        private IntPtr loadFunc(string name)
        {
            IntPtr pAddressOfFunctionToCall = GetProcAddress(hSoftekBarcodeDLL, name);

            if (pAddressOfFunctionToCall == IntPtr.Zero)
            {
                throw new System.DllNotFoundException("Failed to load function " + name);
            }

            return pAddressOfFunctionToCall;
        }


        public BarcodeReader(string rootFolder)
        {
            hBarcode = hSoftekBarcodeDLL = IntPtr.Zero;
            encoding = new System.Text.UTF8Encoding();
            if (loadLibs(rootFolder))
            {
                initBarcodeReader();
            }
        }

        public BarcodeReader()
        {
            hBarcode = hSoftekBarcodeDLL = IntPtr.Zero;
            encoding = new System.Text.UTF8Encoding();
            if (loadLibs(""))
            {
                initBarcodeReader();
            }
        }

        // This function first tries calling LoadLibrary for normal ansi paths
        // and then tries LoadLibraryEx if that fails to load a unicode type path
        // Both are used here because I can't get LoadLibraryEx to work on 
        private IntPtr myLoadLibrary(string dllName)
        {
            IntPtr retval ;

            if ((retval = LoadLibrary(StrToByteArray(dllName))) != IntPtr.Zero)
            {
                return retval ;
            }

            LoadLibraryFlags flags = LoadLibraryFlags.LOAD_LIBRARY_SEARCH_DEFAULT_DIRS |
                                     LoadLibraryFlags.LOAD_LIBRARY_SEARCH_DLL_LOAD_DIR;

            if ((retval = LoadLibraryEx(dllName, IntPtr.Zero, flags)) != IntPtr.Zero)
            {

                return retval;
            }

            return IntPtr.Zero;
        }

        private bool loadLibs(string rootFolder)
        {
            dllLoaded = false;
            pdflib = "DebenuPDFLibraryDLL.dll";

            if (IntPtr.Size == 8)
                pdflib = "DebenuPDFLibrary64DLL.dll";

            string dllname = "SoftekBarcodeDLL.dll";

            if (IntPtr.Size == 8)
            {
                dllname = "SoftekBarcode64DLL.dll";
            }

            if (rootFolder.Length > 0)
            {
                rootFolder += "\\";
            }

            //Console.WriteLine(rootFolder + dllname + "\r\n");

            if ((hSoftekBarcodeDLL = myLoadLibrary(rootFolder + dllname)) == IntPtr.Zero)
            {
                rootFolder += "bin\\";
                if ((hSoftekBarcodeDLL = myLoadLibrary(rootFolder + dllname)) == IntPtr.Zero)
                {
                    //Console.WriteLine("LoadLibrary failed\r\n");
                    throw new System.DllNotFoundException("Failed to load " + dllname + " from " + rootFolder);
                }
            }

            pdfLibHandle = myLoadLibrary(rootFolder + pdflib);

            // These are optional and only required if using the VeryPDF render tool
            myLoadLibrary(rootFolder + "cimage.dll");
            myLoadLibrary(rootFolder + "VeryPDFLibrary.dll");




/*
            if ((hSoftekBarcodeDLL = LoadLibraryEx(rootFolder + dllname, IntPtr.Zero, flags)) == IntPtr.Zero)
            {
                rootFolder += "bin\\";
                if ((hSoftekBarcodeDLL = LoadLibraryEx(rootFolder + dllname, IntPtr.Zero, flags)) == IntPtr.Zero)
                {
                    //Console.WriteLine("LoadLibrary failed\r\n");
                    throw new System.DllNotFoundException("Failed to load " + dllname + " from " + rootFolder);
                    return false;
                }
            }

            pdfLibHandle = LoadLibraryEx(rootFolder + pdflib, IntPtr.Zero, flags);

            // These are optional and only required if using the VeryPDF render tool
            LoadLibraryEx(rootFolder + "cimage.dll", IntPtr.Zero, flags);
            LoadLibraryEx(rootFolder + "VeryPDFLibrary.dll", IntPtr.Zero, flags);
*/


            loadDLLFuncs();
            dllLoaded = true;

            return true;
        }

 
 
        
        private void initBarcodeReader()
        {
            int fl;

            message = "";
            Pattern = "";
            LicenseKey = "";
            TifSplitPath = "";
            TifSplitMode = 0;
            build = "180206 01";

            try
            {
                fl = getFunctionalityLevel();
            }

            catch (System.EntryPointNotFoundException exc)
            {
                throw new System.DllNotFoundException("SoftekBarcode.dll does not support required functionality for this class (missing getFunctionalityLevel)"); 
            }

            if (fl < 4)
            {
                throw new System.DllNotFoundException("SoftekBarcode.dll does not support required functionality for this class: level = " + fl.ToString());
            }

            hBarcode = mtCreateBarcodeInstance();
            GetProperties();
            LoadXMLSettings("<FilePathEncoding>1</>");
            Photometric = true;
        }

        ~ BarcodeReader()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposeManagedResources)
        {
            // process only if mananged and unmanaged resources have
            // not been disposed of.
            if (!this.disposed)
            {
                disposed = true;
                
                // dispose unmanaged resources
                // Trace.WriteLine("BarcodeReader: Disposing unmanaged resouces");

                if (hBarcode != IntPtr.Zero)
                    mtDestroyBarcodeInstance(hBarcode);


                if (hSoftekBarcodeDLL != IntPtr.Zero)
                    FreeLibrary(hSoftekBarcodeDLL);

                if (pdfLibHandle != null)
                {
                    FreeLibrary(pdfLibHandle);
                }

                if (disposeManagedResources)
                {
                    //Trace.WriteLine("BarcodeReader: Disposing managed resources");
                    pdfLibHandle = IntPtr.Zero;
                    encoding = null;
                }

                disposed = true;
            }
            else
            {
                ;// Trace.WriteLine("BarcodeReader: Resources already disposed");
            }
        }

        private byte[] StrToByteArray(string str)
        {
            byte[] b = System.Text.Encoding.UTF8.GetBytes(str);
            // Add null character
            System.Array.Resize(ref b, b.Length + 1);
            b[b.Length - 1] = 0;
            return b;
        }

        public int GetProgress()
        {
            return mtGetProgress(hBarcode);
        }



        public int GetLastError()
        {
            return mtGetLastError(hBarcode);
        }

        public int GetLastWarning()
        {
            return mtGetLastWarning(hBarcode);
        }


        public int GetLastWinError()
        {
            return mtGetLastWinError(hBarcode);
        }

        private bool testFileAccess(string filePath)
        {
            try
            {
                File.Open(filePath, FileMode.Open, FileAccess.Read).Dispose();
                return true;
            }
            catch (IOException)
            {
                return false;
            }
        }

		public int ScanBarCode(string strFilePath)
        {
            if (strFilePath.EndsWith(".pdf", StringComparison.CurrentCultureIgnoreCase))
            {
                if (pdfLibHandle == IntPtr.Zero)
                    throw new System.DllNotFoundException("The following dll file needed for processing pdf documents could not be loaded: " + pdflib);
            }
		    SetProperties() ;
            //if (!testFileAccess(strFilePath))
            //    return -999;

            return mtScanBarCode(hBarcode, StrToByteArray(strFilePath)) ;
        }

        public int ScanBarCodeInBackground(string strFilePath)
        {
            if (strFilePath.EndsWith(".pdf", StringComparison.CurrentCultureIgnoreCase))
            {
                if (pdfLibHandle == IntPtr.Zero)
                    throw new System.DllNotFoundException("The following dll file needed for processing pdf documents could not be loaded: " + pdflib);
            }
            SetProperties();
            return mtScanBarCodeInBackground(hBarcode, StrToByteArray(strFilePath));
        }

        public int ScanBarCodeFromByteArrayInBackground(byte[] data)
        {
            SetProperties();
            return mtScanBarCodeFromStringInBackground(hBarcode, data, (uint) data.Length);
        }

        public int ScanBarCodeAbort()
        {
            return mtScanBarCodeAbort(hBarcode);
        }

        public int GetScanExitCode()
        {
            return mtGetScanExitCode(hBarcode);
        }

        // -1 means infinite, 0 is just a check 
        public int ScanBarCodeWait(int milliSeconds)
        {
            return mtScanBarCodeWait(hBarcode, milliSeconds); 
        }

        public int GetBarCodeCount()
        {
            return mtGetBarCodeCount(hBarcode);
        }

        public int GetBlankPageCount()
        {
            return mtGetBlankPageCount(hBarcode);
        }

        public int GetBlankPage(int nBlankPage)
        {
            return mtGetBlankPage(hBarcode, (short) nBlankPage);
        }

        public int GetBarStringQualityScore(int nBarcode)
        {
            return mtGetBarStringQualityScore(hBarcode, (short)nBarcode);
        }

        public byte[] GetBarStringBytes(int nBarcode)
        {
            string ansiStr = System.Runtime.InteropServices.Marshal.PtrToStringAnsi(mtGetBarString(hBarcode, (short)nBarcode));
            System.Text.Encoding iso = System.Text.Encoding.GetEncoding("ISO-8859-1");
            return iso.GetBytes(ansiStr);
        }


        public byte[] GetRawBarStringBytes(int nBarcode)
        {
            // Get the required size 
            int sz = mtGetRawBarString(hBarcode, (short)nBarcode, null, 0);

            if (sz < 0) return new byte[0];

            // Now get the bytes for real
            byte[] b = new byte[sz];
            sz = mtGetRawBarString(hBarcode, (short)nBarcode, b, b.Length);

            if (sz < 0) return new byte[0];

            return b;
        }

		public string GetBarString(int nBarcode)
        {
            if (Encoding == 3) // UTF-8 encoding (the default)
            {
                int bufSize = 10000;

                byte[] buffer = new byte[bufSize];

                int l = mtGetBarStringA(hBarcode, (short)nBarcode, buffer, bufSize);

                if (l > 0)
                    return System.Text.Encoding.UTF8.GetString(buffer, 0, l);
                else
                    return "";
            }
            else
            {
                return System.Runtime.InteropServices.Marshal.PtrToStringAnsi(mtGetBarString(hBarcode, (short)nBarcode));
            }
        }


		public string GetBarStringType(int nBarcode)
        {
            return System.Runtime.InteropServices.Marshal.PtrToStringAnsi(mtGetBarStringType(hBarcode, (short) nBarcode));
        }

		public System.Drawing.Rectangle GetBarStringRect(int nBarcode)
        {
            int tlx, tly, brx, bry ;
            tlx = mtGetBarStringTlx(hBarcode, (short) nBarcode);
            tly = mtGetBarStringTly(hBarcode, (short)nBarcode);
            brx = mtGetBarStringBrx(hBarcode, (short)nBarcode);
            bry = mtGetBarStringBry(hBarcode, (short)nBarcode);

            System.Drawing.Rectangle rect = new Rectangle(tlx, tly, brx - tlx, bry - tly);
            return rect ;
        }

		public int SetScanRect(System.Drawing.Rectangle rect)
        {
            return SetScanRect(rect, 0);
        }

		public int SetScanRect(System.Drawing.Rectangle rect, int MappingMode)
        {
            if (mtSetScanRectA(hBarcode, (Int32)rect.Left, (Int32)rect.Top, (Int32)rect.Right, (Int32)rect.Bottom, (short)MappingMode))
                return 1;
            return 0;
        }

		public int GetBarStringDirection(int nBarCode)
        {
            return mtGetBarStringDirection(hBarcode, (short) nBarCode) ;
        }

		public int GetBarStringPage(int n)
        {
            return mtGetBarStringPage(hBarcode, (short)n);
        }

		public int ScanBarCodeFromBitmap(System.Drawing.Bitmap bmp)
        {
            SetProperties();

            System.Drawing.Imaging.PixelFormat pixelFormat = bmp.PixelFormat;

            short bitsPerPel;

            if (bmp.PixelFormat == System.Drawing.Imaging.PixelFormat.Format1bppIndexed)
            {
                bitsPerPel = 1;
            }
            else if (bmp.PixelFormat == System.Drawing.Imaging.PixelFormat.Format8bppIndexed)
            {
                bitsPerPel = 8;
            }
            else
            {
                bitsPerPel = 24;
                pixelFormat = System.Drawing.Imaging.PixelFormat.Format24bppRgb;
            }
            
            System.Drawing.Rectangle rect = new System.Drawing.Rectangle(0, 0, bmp.Width, bmp.Height);
            System.Drawing.Imaging.BitmapData bmData = bmp.LockBits(rect, System.Drawing.Imaging.ImageLockMode.ReadOnly,pixelFormat) ;

            int retval ;

            if ((bmData.Stride > 0) && ((bmData.Stride % 4) == 0))
            {
                retval = mtScanBarCodeFromBitmapA(hBarcode, 0, bmData.Width, bmData.Height, bmData.Stride, 1, bitsPerPel, bmData.Scan0);
                bmp.UnlockBits(bmData);
            }
            else
            {
                IntPtr hBitmap = bmp.GetHbitmap();

                if (hBitmap == IntPtr.Zero)
                {
                    bmp.UnlockBits(bmData);
                    return -1;
                }

                retval = mtScanBarCodeFromBitmap(hBarcode, hBitmap);

                DeleteObject(hBitmap);
            }
            return retval;
        }

		public int ScanBarCodeFromBitmap(IntPtr hBitmap)
        {
            SetProperties();
            return mtScanBarCodeFromBitmap(hBarcode, hBitmap);
        }

        public int ScanBarCodeFromByteArray(byte[] data)
        {
            SetProperties();
            return mtScanBarCodeFromString(hBarcode, data, (uint) data.Length);
        }

		public int ScanBarCodeFromDIB (IntPtr hDIB)
        {
            SetProperties();
            return mtScanBarCodeFromDIB(hBarcode, hDIB);
        }

		public bool ProcessXML(string strInputFilePath, string strOutputFilePath)
        {
            return ProcessXML(strInputFilePath, strOutputFilePath, true) ;
        }

        public bool ProcessXML(string strInputFilePath, string strOutputFilePath, bool silent)
        {
            SetProperties();
            bool retval = Convert.ToBoolean(mtProcessXML(hBarcode, StrToByteArray(strInputFilePath), StrToByteArray(strOutputFilePath), silent));
            GetProperties();
            return retval;
        }

		public bool ExportXMLSettings(string strFilePath)
        {
            SetProperties();
            bool retval = Convert.ToBoolean(mtExportXMLSettings(hBarcode, StrToByteArray(strFilePath)));
            GetProperties();
            return retval;
        }

		public bool SaveResults(string strFilePath)
        {
            SetProperties();
            bool retval = Convert.ToBoolean(mtSaveResults(hBarcode, StrToByteArray(strFilePath)));
            GetProperties();
            return retval;
        }

		public bool LoadXMLSettings(string strFilePath)
        {
            return LoadXMLSettings(strFilePath, true) ;
        }

        public bool LoadXMLSettings(string strFilePath, bool silent)
        {
            SetProperties();
            bool retval = Convert.ToBoolean(mtLoadXMLSettings(hBarcode, StrToByteArray(strFilePath), silent));
            GetProperties();
            return retval;
        }

        private void GetProperties()
        {
            MinOccurrence = mtGetMinOccurrence(hBarcode);
            LineJump = mtGetLineJumpA(hBarcode);
            PrefOccurrence = mtGetPrefOccurrence(hBarcode);
            MultipleRead = Convert.ToBoolean(mtGetMultipleRead(hBarcode));
            NoiseReduction = mtGetNoiseReduction(hBarcode);
            Despeckle = mtGetDespeckle(hBarcode);
            QuietZoneSize = mtGetQuietZoneSize(hBarcode);
            ReadCode39 = Convert.ToBoolean(mtGetReadCode39(hBarcode));
            ReadCode93 = Convert.ToBoolean(mtGetReadCode93(hBarcode));
            ReadPDF417 = Convert.ToBoolean(mtGetReadPDF417(hBarcode));
            ReadCode128 = Convert.ToBoolean(mtGetReadCode128(hBarcode));
            ReadCode25 = Convert.ToBoolean(mtGetReadCode25(hBarcode));
            ReadCode25ni = Convert.ToBoolean(mtGetReadCode25ni(hBarcode));
            ReadEAN8 = Convert.ToBoolean(mtGetReadEAN8(hBarcode));
            AllowDuplicateValues = Convert.ToBoolean(mtGetAllowDuplicateValues(hBarcode));
            ReadEAN13 = Convert.ToBoolean(mtGetReadEAN13(hBarcode));
            ReadUPCA = Convert.ToBoolean(mtGetReadUPCA(hBarcode));
            ReadUPCE = Convert.ToBoolean(mtGetReadUPCE(hBarcode));
            ReadCodabar = Convert.ToBoolean(mtGetReadCodabar(hBarcode));
            ReadPatchCodes = Convert.ToBoolean(mtGetReadPatchCodes(hBarcode));
            ConvertUPCEToEAN13 = Convert.ToBoolean(mtGetConvertUPCEToEAN13(hBarcode));
            ShowCheckDigit = Convert.ToBoolean(mtGetShowCheckDigit(hBarcode));
            Code39NeedStartStop = Convert.ToBoolean(mtGetCode39NeedStartStop(hBarcode));
            ColorThreshold = mtGetColorThreshold(hBarcode);
            UseOverSampling = Convert.ToBoolean(mtGetUseOverSampling(hBarcode));
            ScanDirection = mtGetScanDirection(hBarcode);
            MinLength = mtGetMinLength(hBarcode);
            Min2DLength = mtGetMin2DLength(hBarcode);
            MaxLength = mtGetMaxLength(hBarcode);
            SkewTolerance = mtGetSkewTolerance(hBarcode);
            Code39Checksum = Convert.ToBoolean(mtGetCode39Checksum(hBarcode));
            ExtendedCode39 = Convert.ToBoolean(mtGetExtendedCode39(hBarcode));
            ErrorCorrection = Convert.ToBoolean(mtGetErrorCorrection(hBarcode));
            MedianFilter = Convert.ToBoolean(mtGetMedianFilter(hBarcode));
            ReadNumeric = Convert.ToBoolean(mtGetReadNumeric(hBarcode));
            MinSeparation = mtGetMinSeparation(hBarcode);
            SkewLineJump = mtGetSkewLineJump(hBarcode);
            GammaCorrection = mtGetGammaCorrection(hBarcode);
            Encoding = mtGetEncoding(hBarcode);
            Code25Checksum = Convert.ToBoolean(mtGetCode25Checksum(hBarcode));
            PageNo = mtGetPageNo(hBarcode);
            PdfBpp = mtGetPdfBpp(hBarcode);
            PdfDpi = mtGetPdfDpi(hBarcode);
            Photometric = mtGetPhotometric(hBarcode);
            MinSpaceBarWidth = mtGetMinSpaceBarWidth(hBarcode);
            ReadMicroPDF417 = Convert.ToBoolean(mtGetReadMicroPDF417(hBarcode));
            ReadShortCode128 = Convert.ToBoolean(mtGetReadShortCode128(hBarcode));
            ShortCode128MinLength = mtGetShortCode128MinLength(hBarcode);
            ReadDataMatrix = Convert.ToBoolean(mtGetReadDataMatrix(hBarcode));
            ReadQRCode = Convert.ToBoolean(mtGetReadQRCode(hBarcode));
            ReadDatabar = Convert.ToBoolean(mtGetReadDatabar(hBarcode));
            DatabarOptions = mtGetDatabarOptions(hBarcode);
            ColorProcessingLevel = mtGetColorProcessingLevel(hBarcode);
            PdfImageOnly = Convert.ToBoolean(mtGetPdfImageOnly(hBarcode));
            PdfImageExtractOptions = mtGetPdfImageExtractOptions(hBarcode);
            PdfImageRasterOptions = mtGetPdfImageRasterOptions(hBarcode);
            UseFastScan = mtGetUseFastScan(hBarcode);
            BarcodesAtTopOfPage = mtGetBarcodesAtTopOfPage(hBarcode);
            FastScanLineJump = mtGetFastScanLineJump(hBarcode);
            MaxBarcodesPerPage = mtGetMaxBarcodesPerPage(hBarcode);
            SkewedDatamatrix = mtGetSkewedDatamatrix(hBarcode);
            TimeOut = mtGetTimeOut(hBarcode);
            SkewedLinear = mtGetSkewedLinear(hBarcode);
            ReportUnreadBarcodes = mtGetReportUnreadBarcodes(hBarcode);
            ShowCodabarStartStop = mtGetShowCodabarStartStop(hBarcode);
            Pattern = System.Runtime.InteropServices.Marshal.PtrToStringAnsi(mtGetPattern(hBarcode));
            MaxThreads = mtGetMaxThreads(hBarcode);
            RotateBy45IfNoBarcode = mtGetRotateBy45IfNoBarcode(hBarcode);
            Code128DebugMode = mtGetCode128DebugMode(hBarcode);
            Code128Lenient = mtGetCode128Lenient(hBarcode);
            Code128SearchLevel = mtGetCode128SearchLevel(hBarcode);
            MedianFilterBias = mtGetMedianFilterBias(hBarcode);
            Pdf417Debug = mtGetPdf417Debug(hBarcode);
            Pdf417ChannelMode = mtGetPdf417ChannelMode(hBarcode);
            PdfLocking = mtGetPdfLocking(hBarcode);
            QrCodeReadInverted = mtGetQrCodeReadInverted(hBarcode);
            QuotedPrintableCharSet = mtGetQuotedPrintableCharSet(hBarcode);
            WeightLongerBarcodes = mtGetWeightLongerBarcodes(hBarcode);
            Code39MaxRatioPcnt = mtGetCode39MaxRatioPcnt(hBarcode);
            DataMatrixFinderGapTolerance = mtGetDataMatrixFinderGapTolerance(hBarcode);
            DataMatrixAutoUTF8 = mtGetDataMatrixAutoUTF8(hBarcode);
            PatchCodeMinOccurrence = mtGetPatchCodeMinOccurrence(hBarcode);
            Pdf417AutoUTF8 = mtGetPdf417AutoUTF8(hBarcode);
            Pdf417MacroEscapeBackslash = mtGetPdf417MacroEscapeBackslash(hBarcode);
            QRCodeAutoUTF8 = mtGetQRCodeAutoUTF8(hBarcode);
            QRCodeAutoMedianFilter = mtGetQRCodeAutoMedianFilter(hBarcode);
            UseOldCode128Algorithm = mtGetUseOldCode128Algorithm(hBarcode);
            TifSplitMode = mtGetTifSplitMode(hBarcode);
            TifSplitPath = System.Runtime.InteropServices.Marshal.PtrToStringAnsi(mtGetTifSplitPath(hBarcode));
            UseRunCache = mtGetUseRunCache(hBarcode);
            DeskewMode = mtGetDeskewMode(hBarcode);
            PdfPassword = System.Runtime.InteropServices.Marshal.PtrToStringAnsi(mtGetPdfPassword(hBarcode));
            DataMatrixSearchLevel = mtGetDataMatrixSearchLevel(hBarcode);
            DataMatrixRectangleSupport = mtGetDataMatrixRectangleSupport(hBarcode);
            ReadEAN13Supplemental = mtGetReadEAN13Supplemental(hBarcode);
            IndexBlankPages = mtGetIndexBlankPages(hBarcode);
            BlankPageStandardDeviation = mtGetBlankPageStandardDeviation(hBarcode);
            BlankPageMargin = mtGetBlankPageMargin(hBarcode);

        }

	    private void SetProperties()
	    {
		    mtSetMinOccurrence(hBarcode, (short) MinOccurrence);
            mtSetLineJumpA(hBarcode, (short)LineJump);
            mtSetPrefOccurrence(hBarcode, (short)PrefOccurrence);
            mtSetMultipleRead(hBarcode, MultipleRead);
		    mtSetNoiseReduction(hBarcode, (short) NoiseReduction);
		    mtSetDespeckle(hBarcode, Despeckle);
		    mtSetQuietZoneSize(hBarcode, (short) QuietZoneSize);
            mtSetReadCode39(hBarcode, ReadCode39);
            mtSetReadCode93(hBarcode, ReadCode93);
            mtSetReadPDF417(hBarcode, ReadPDF417);
            mtSetReadCode128(hBarcode, ReadCode128);
            mtSetReadCode25(hBarcode, ReadCode25);
            mtSetReadCode25ni(hBarcode, ReadCode25ni);
            mtSetReadEAN8(hBarcode, ReadEAN8);
            mtSetAllowDuplicateValues(hBarcode, AllowDuplicateValues);
            mtSetReadEAN13(hBarcode, ReadEAN13);
            mtSetReadUPCA(hBarcode, ReadUPCA);
            mtSetReadUPCE(hBarcode, ReadUPCE);
            mtSetReadCodabar(hBarcode, ReadCodabar);
            mtSetReadPatchCodes(hBarcode, ReadPatchCodes);
            mtSetConvertUPCEToEAN13(hBarcode, ConvertUPCEToEAN13);
            mtSetShowCheckDigit(hBarcode, ShowCheckDigit);
            mtSetCode39NeedStartStop(hBarcode, Code39NeedStartStop);
		    mtSetColorThreshold(hBarcode, (short) ColorThreshold);
            mtSetUseOverSampling(hBarcode, UseOverSampling);
		    mtSetScanDirection(hBarcode, (short) ScanDirection);
		    mtSetMinLength(hBarcode, (short) MinLength);
            mtSetMin2DLength(hBarcode, (short)Min2DLength);
            mtSetMaxLength(hBarcode, (short)MaxLength);
		    mtSetSkewTolerance(hBarcode, (short) SkewTolerance);
            mtSetCode39Checksum(hBarcode, Code39Checksum);
            mtSetExtendedCode39(hBarcode, ExtendedCode39);
            mtSetErrorCorrection(hBarcode, ErrorCorrection);
            mtSetMedianFilter(hBarcode, MedianFilter);
            mtSetReadNumeric(hBarcode, ReadNumeric);
		    mtSetMinSeparation(hBarcode, (short) MinSeparation);
		    mtSetMinSpaceBarWidth(hBarcode, (short) MinSpaceBarWidth);
		    mtSetSkewLineJump(hBarcode, (short) SkewLineJump);
		    mtSetGammaCorrection(hBarcode, (short) GammaCorrection);
		    mtSetEncoding(hBarcode, (short) Encoding);
		    mtSetCode25Checksum(hBarcode, Code25Checksum);
		    mtSetPageNo(hBarcode, (short) PageNo);
		    mtSetPdfBpp(hBarcode, (short) PdfBpp);
		    mtSetPdfDpi(hBarcode, (short) PdfDpi);
		    mtSetPhotometric(hBarcode, Photometric);
            mtSetReadMicroPDF417(hBarcode, ReadMicroPDF417);
            mtSetReadShortCode128(hBarcode, ReadShortCode128);
		    mtSetShortCode128MinLength(hBarcode, (short) ShortCode128MinLength);
            mtSetReadDataMatrix(hBarcode, ReadDataMatrix);
            mtSetReadQRCode(hBarcode, ReadQRCode);
            mtSetReadDatabar(hBarcode, ReadDatabar);
		    mtSetDatabarOptions(hBarcode, (short) DatabarOptions);
		    mtSetColorProcessingLevel(hBarcode, (short) ColorProcessingLevel);
            mtSetTifSplitMode(hBarcode, (short) TifSplitMode);
            mtSetPdfImageOnly(hBarcode, PdfImageOnly);
            mtSetPdfImageExtractOptions(hBarcode, (short)PdfImageExtractOptions);
            mtSetPdfImageRasterOptions(hBarcode, (short)PdfImageRasterOptions);
            mtSetUseFastScan(hBarcode, UseFastScan);
            mtSetBarcodesAtTopOfPage(hBarcode, BarcodesAtTopOfPage);
            mtSetFastScanLineJump(hBarcode, (short) FastScanLineJump);
            mtSetMaxBarcodesPerPage(hBarcode, (short) MaxBarcodesPerPage);
            mtSetSkewedDatamatrix(hBarcode, SkewedDatamatrix);
            mtSetTimeOut(hBarcode, TimeOut);
            mtSetSkewedLinear(hBarcode, SkewedLinear);
            mtSetReportUnreadBarcodes(hBarcode, (short) ReportUnreadBarcodes);
            mtSetShowCodabarStartStop(hBarcode, (short) ShowCodabarStartStop);
            mtSetMaxThreads(hBarcode, (short)MaxThreads);
            mtSetRotateBy45IfNoBarcode(hBarcode, RotateBy45IfNoBarcode);
            mtSetCode128DebugMode(hBarcode, (short)Code128DebugMode);
            mtSetCode128Lenient(hBarcode, Code128Lenient);
            mtSetCode128SearchLevel(hBarcode, (short)Code128SearchLevel);
            mtSetMedianFilterBias(hBarcode, (short)MedianFilterBias);
            mtSetPdf417Debug(hBarcode, Pdf417Debug);
            mtSetPdf417ChannelMode(hBarcode, (short)Pdf417ChannelMode);
            mtSetPdfLocking(hBarcode, (short)PdfLocking);
            mtSetQrCodeReadInverted(hBarcode, QrCodeReadInverted);
            mtSetQuotedPrintableCharSet(hBarcode, (short)QuotedPrintableCharSet);
            mtSetWeightLongerBarcodes(hBarcode, WeightLongerBarcodes);
            mtSetCode39MaxRatioPcnt(hBarcode, (short)Code39MaxRatioPcnt);
            mtSetDataMatrixFinderGapTolerance(hBarcode, (short)DataMatrixFinderGapTolerance);
            mtSetDataMatrixAutoUTF8(hBarcode, DataMatrixAutoUTF8);
            mtSetPatchCodeMinOccurrence(hBarcode, (short)PatchCodeMinOccurrence);
            mtSetPdf417AutoUTF8(hBarcode, Pdf417AutoUTF8);
            mtSetPdf417MacroEscapeBackslash(hBarcode, Pdf417MacroEscapeBackslash);
            mtSetQRCodeAutoUTF8(hBarcode, QRCodeAutoUTF8);
            mtSetQRCodeAutoMedianFilter(hBarcode, QRCodeAutoMedianFilter);
            mtSetUseOldCode128Algorithm(hBarcode, UseOldCode128Algorithm);
            mtSetUseRunCache(hBarcode, UseRunCache);
            mtSetDeskewMode(hBarcode, (short)DeskewMode);
            mtSetDataMatrixSearchLevel(hBarcode, (short)DataMatrixSearchLevel);
            mtSetDataMatrixRectangleSupport(hBarcode, (short)DataMatrixRectangleSupport);
            mtSetReadEAN13Supplemental(hBarcode, ReadEAN13Supplemental);
            mtSetIndexBlankPages(hBarcode, IndexBlankPages);
            mtSetBlankPageMargin(hBarcode, BlankPageMargin);
            mtSetBlankPageStandardDeviation(hBarcode, BlankPageStandardDeviation);

    	
            byte[] byteStr = StrToByteArray(Pattern);
		    mtSetPattern(hBarcode, byteStr) ;

            byteStr = StrToByteArray(TifSplitPath) ;
		    mtSetTifSplitPath(hBarcode, byteStr) ;

            byteStr = StrToByteArray(LicenseKey) ;
            mtSetLicenseKey(hBarcode, byteStr) ;

            byteStr = StrToByteArray(PdfPassword);
            mtSetPdfPassword(hBarcode, byteStr);

		    mtSetTifSplitMode(hBarcode, (short) TifSplitMode) ;
	    }

    }
}
